@@ -8,7 +8,7 @@ import threading
 
 logging.basicConfig(stream=sys.stdout, level=os.environ.get("LOGLEVEL", logging.INFO))
 
-__version__ = "1.8.0"
+__version__ = "1.9.0"
 
 logger = logging.getLogger(__name__)
 
@@ -94,7 +94,7 @@ class RepeatedJob(object):
         hybridlogger.ha_log(
             self.logger,
             self.hass_api,
-            "INFO",
+            "DEBUG",
             f"new poll in {self.calculated_interval} seconds at {self.new_report_expected_at.isoformat()}.",
         )
         self.start()
